/*#   Copyright 2012 http://scms.pro/
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
*/

/*
 * SCMS Hydra Library
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>

#define READ 0
#define WRITE 1

pid_t popen2(const char *command, int *infp, int *outfp)
{
    int p_stdin[2], p_stdout[2];
    pid_t pid;

    if (pipe(p_stdin) != 0 || pipe(p_stdout) != 0)
        return -1;

    pid = fork();

    if (pid < 0)
        return pid;
    else if (pid == 0)
    {
        close(p_stdin[WRITE]);
        dup2(p_stdin[READ], READ);
        close(p_stdout[READ]);
        dup2(p_stdout[WRITE], WRITE);
	close(p_stdout[READ]);
	close(p_stdin[WRITE]);

        execl("/bin/sh", "sh", "-c", command, NULL);
        perror("execl");
        exit(1);
    }

    if (infp == NULL)
        close(p_stdin[WRITE]);
    else
        *infp = p_stdin[WRITE];

    if (outfp == NULL)
        close(p_stdout[READ]);
    else
        *outfp = p_stdout[READ];

    close(p_stdin[READ]);
    close(p_stdout[WRITE]);
    return pid;
}

char *gnu_dirname(char *path)
{
    char *base = strrchr(path, '/');
    if (base) {
      *base = '\0';
    }
    return path;
}


char *prefix;
char *prefix_lib;

#define ERROR(args...) do{ if( verbose) fprintf( stderr, args); }while(0);

static int ctlport = 9010;
static int sock;

static char *root_of_scit, *manager_var, *mvsbin, *mvsnodes;

#include "utils.h"
#include "privatekey.h"

static int openserver( void) {

    struct sockaddr_in addr;
    int yes = 1;

    memset( &addr, 0, sizeof( addr));
    addr.sin_family = AF_INET;
    addr.sin_port = htons( ctlport);
    
    if( (sock = socket( AF_INET, SOCK_STREAM, 0)) < 0) {
	perror( "socket()");
	return 1;
    }
    
    if( setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof( int)) < 0) {
	perror( "setsockopt");
	return 1;
    }
    
    if( bind( sock, (struct sockaddr *)&addr, sizeof( addr)) < 0) {
	perror( "bind()");
	return 1;
    }

    if( listen( sock, 5) < 0) {
	perror( "listen()");
	return 1;
    }
    return 0;
}

static void sig_CHLD( int sig) {

    int stat;

    while( waitpid( -1, &stat, WNOHANG) > 0);
} 


static void help( void) {

    fprintf( stderr, "Usage:\n\
-v   verbose mode\n");
    exit( 1);
}

static void do_exec( int fd, char *cmd) {

    char str[ 4194304];
    int l;
 
    if( verbose) fprintf( stderr, "executing: %s\n", cmd);

    int infp, outfp;
    char buf[128];

    if (popen2(cmd, &infp, &outfp) <= 0) {
      snprintf( str, sizeof( cmd), "errno %d, %s", errno, strerror( errno));
      write_str( fd, str, 0);
      return;
    }
    
    if( verbose) fprintf( stderr, "command pipe opened\n");    
 
    while( (l = read_str( fd, str, sizeof( str)))) {
        if( l <= 0) 
          break;
        if( verbose) fprintf( stderr, "Get from socket: '%s'[ %d]\n", str, l);
        usleep(2);
        if(! write( infp, str,l)) {
		      perror( "write");
          break;
        }
    }
    close(infp);
    if( verbose) fprintf( stderr, "input pipe closed\n");
 
    while( l = read(outfp, str, sizeof(str))) {
        if( l <= 0) 
          break;
        if( verbose) fprintf( stderr, "Get from command pipe: '%s'[ %d]\n", str, l);
	      if ( write_str(fd,str,l) <= 0)
          break;
    }

    if( verbose) fprintf( stderr, "Done\n", str, l);
    close( outfp);
    if( verbose) fprintf( stderr, "output pipe closed\n");
}

void proceed( int fd) {

    char str[ 1024], *s, cmd[ 4096];
    int len;
    struct passwd *passwd;

    if( write( fd, "Yes?\n", 6) != 6) {
	fprintf( stderr, "Can\'t Write \"Yes\"\n");
        goto handshake;
    }
    if( read( fd, str, sizeof( str)) != 6) {
        fprintf( stderr, "'%s'[ %d]\n", str, strlen( str));
        goto handshake;
    }
    if( strcmp( str, "hydra")) {
        fprintf( stderr, "'%s'[ %d]\n", str, strlen( str));
        goto handshake;
    }

    if( write_str( fd, HANDSHAKE1, HANDSHAKE1LEN) != HANDSHAKE1LEN)
        goto handshake;
    if( read_str( fd, str, sizeof( str)) != HANDSHAKE2LEN)
        goto handshake;
    if( strcmp( str, HANDSHAKE2))
        goto handshake;

    if( !write_ok( fd))
        goto handshake;

    if( (len = read_str( fd, str, sizeof( str))) < 0)
        goto handshake;

    if( memcmp( str, "user ", 5))
        goto handshake;

    s = str + 5;
    if( !(passwd = getpwnam( s))) {
        if( verbose) fprintf( stderr, "username %s not found in user database: %s\n", s, strerror( errno));
        exit( 1);
    }

    if( passwd->pw_uid < 500) {
        if( verbose) fprintf( stderr, "user %s uid %d is not allowed to use interface\n", s, passwd->pw_uid);
        exit( 1);
    }

    if( setuid( passwd->pw_uid)) {
        if( verbose) fprintf( stderr, "err setuid( %s): %s", s, strerror( errno));
        exit( 1);
    }

    if( !write_ok( fd))
        goto handshake;

    clearenv();
    setenv( "USER", passwd->pw_name, 1);
    setenv( "HOME", passwd->pw_dir, 1);
    setenv( "SHELL", passwd->pw_shell, 1);
    setenv( "PATH", "/bin:/usr/bin", 1);
    setenv( "TMP", "/tmp", 1);
    setenv( "TMPDIR", "/tmp", 1);

    chdir( passwd->pw_dir);
    
    if( read_str( fd, cmd, sizeof( cmd)) < 0)
        goto handshake;
    if( !write_ok( fd))
        goto handshake;
    char *exec;
    asprintf(&exec,"sh -l -c '%s/%s'",prefix_lib,cmd);
    if (verbose) fprintf(stderr, "cmd: %s, exec: %s\n", cmd, exec);

    do_exec( fd, exec);

    write_str( fd, ".DONE.", 0);

done:
    exit( 0);
handshake:
    ERROR( "handshake failed\n");
    exit( 1);
}

int main( int ac, char **av) {

  char* ptr;
  char actualpath [1024];
  ptr = realpath(av[0], actualpath);
  char *p = strdup(actualpath);
  prefix = (char*) gnu_dirname(gnu_dirname(p));
//  fprintf( stderr, "prefix: %s\n", prefix);
//  return 0;

  asprintf(&prefix_lib, "%s/libexec",prefix);

  char *s;
  struct sockaddr_in addr;
  int addr_len = sizeof( addr), cl;
  pid_t pid;

  ac--; av++;
   
  while( ac) {
    s = *av;
    if( *s != '-') break;
    s++; av++; ac--;
    switch( *s) {
    case 'v':
      verbose++;
      continue;
    default:
      help();
    }
  }

  signal( SIGCHLD, sig_CHLD);
  signal( SIGPIPE, SIG_IGN);
    
  openserver();

  if( !verbose)
    daemon( 0, 0);
    
    while( 1) {
    
        if( do_select( sock, 0))
            return 90;
	
	if( (cl = accept( sock, (struct sockaddr *)&addr, &addr_len)) < 0) {
	    if( verbose) fprintf( stderr, "err accept(): %s\n", strerror( errno));
	    continue;
        }

        if( !(pid = fork()))
            proceed( cl);

        close( cl);

        if( pid < 0)
            ERROR( "fork(): %s\n", strerror( errno));
    }
    return 0;
}
