Ext.ns('Ext.ux.form');
Ext.ux.form.TriCheckbox = Ext.extend(Ext.form.Checkbox, {
  checked: null,
  valueList: [null, false, true],
  stateClassList: ['x-checkbox-undef', null, 'x-checkbox-checked'],
  overClass: 'x-form-check-over',
  clickClass: 'x-form-check-down',
  triState: true,
  defaultAutoCreate: {
    tag: 'input',
    type: 'hidden',
    autocomplete: 'off'
  },
  initComponent: function () {
    this.value = this.checked;
    Ext.ux.form.TriCheckbox.superclass.initComponent.apply(this, arguments);
    // make a copy before modifying valueList and stateClassList arrays
    this.vList = this.valueList.slice(0);
    this.cList = this.stateClassList.slice(0);
    if (this.triState !== true) {
      // consider 'undefined' value and its class go first in arrays
      this.vList.shift();
      this.cList.shift();
    }
    if (this.overCls !== undefined) {
      this.overClass = this.overCls;
      delete this.overCls;
    }
    this.value = this.normalizeValue(this.value);
  },
  onRender: function (ct, position) {
    Ext.form.Checkbox.superclass.onRender.call(this, ct, position);

    this.innerWrap = this.el.wrap({
      tag: 'span',
      cls: 'x-form-check-innerwrap'
    });
    this.wrap = this.innerWrap.wrap({
      cls: 'x-form-check-wrap'
    });

    this.currCls = this.getCls(this.value);
    this.wrap.addClass(this.currCls);
    if (this.clickClass && !this.disabled && !this.readOnly) this.innerWrap.addClassOnClick(this.clickClass);
    if (this.overClass && !this.disabled && !this.readOnly) this.innerWrap.addClassOnOver(this.overClass);

    this.imageEl = this.innerWrap.createChild({
      tag: 'img',
      src: Ext.BLANK_IMAGE_URL,
      cls: 'x-form-tscheckbox'
    }, this.el);
    if (this.fieldClass) this.imageEl.addClass(this.fieldClass);

    if (this.boxLabel) {
      this.innerWrap.createChild({
        tag: 'label',
        htmlFor: this.el.id,
        cls: 'x-form-cb-label',
        html: this.boxLabel
      });
    }

    // Need to repaint for IE, otherwise positioning is broken
    if (Ext.isIE) {
      this.wrap.repaint();
    }
    this.resizeEl = this.positionEl = this.wrap;
  },
  onResize: function () {
    Ext.form.Checkbox.superclass.onResize.apply(this, arguments);
    if (!this.boxLabel && !this.fieldLabel && this.imageEl) {
      this.imageEl.alignTo(this.wrap, 'c-c');
    }
  },
  initEvents: function () {
    Ext.form.Checkbox.superclass.initEvents.call(this);
    this.mon(this.innerWrap, {
      scope: this,
      click: this.onClick
    });
  },
  onClick: function () {
    if (!this.disabled && !this.readOnly) {
      this.setValue(this.vList[(this.vList.indexOf(this.value) + 1) % this.vList.length]);
    }
  },
  getValue: function () {
    return this.value;
  },
  setValue: function (v) {
    var value = this.value;
    this.value = this.normalizeValue(v);
    if (this.rendered) this.el.dom.value = this.value;

    if (value !== this.value) {
      this.updateView();
      this.fireEvent('check', this, this.value);
      if (this.handler) this.handler.call(this.scope || this, this, this.value);
    }
    return this;
  },
  normalizeValue: function (v) {
    return (v === null || v === undefined) && this.triState ? null : (v === true || (['true', 'yes', 'on', '1']).indexOf(String(v).toLowerCase()) != -1);
  },
  getCls: function (v) {
    var idx = this.vList.indexOf(this.value);
    return idx > -1 ? this.cList[idx] : undefined;
  },
  updateView: function () {
    var cls = this.getCls(this.value);
    if (!this.wrap || cls === undefined) return;

    this.wrap.replaceClass(this.currCls, cls);
    this.currCls = cls;
  }
});
Ext.reg('tricheckbox', Ext.ux.form.TriCheckbox);

Ext.ux.form.TriCheckboxGroup = Ext.extend(Ext.form.CheckboxGroup, {
  defaultType: 'tricheckbox',
		getValue : function(){
				var out = [];
				this.eachItem(function(item){
						out.push(item.getValue());
				});
				return out;
  }
});

Ext.ns('app');

app.AttrWindow = Ext.extend(Ext.Window, {
		modal: true,
		width: 300,
		autoHeight: true,
		title: ~'Атрибути файлів'~,
		closeAction: 'hide',
		iconCls: 'x-action-attributes',
		
		setUp: function(files, path){
				var i, j, d, m, a = [], 
								size = 0, fcount = 0, dcount = 0, 
								modesSum = {1:0, 2:0, 4:0, 10:0, 20:0, 40:0, 100:0, 200:0, 400:0}, // 000
								modesMul = {1:1, 2:1, 4:1, 10:1, 20:1, 40:1, 100:1, 200:1, 400:1}, // 777
								states = {};
				this.files = files;
				for (i = 0; i < files.length; i++) {
						d = files[i].data;
						size += d['is_dir'] ? 0 : d['size'];
						fcount += d['is_dir'] ? 0 : 1;
						dcount += d['is_dir'] ? 1 : 0;
						m = this.fileModeDecode(d['mode']);
						for (j in m) {
							 modesSum[j] += m[j];
							 modesMul[j] = modesMul[j] * m[j];
						}
				}
				
				for (j in modesSum) 
					 states[j] = modesSum[j] * modesMul[j] > 0 ? true : (modesSum[j] > 0 ? null : false); 
				
				this.currentPath = path;
				
				if (dcount > 0)
						a.push(String.format(~'{0} директорій'~, dcount));
				if (fcount > 0)
						a.push(String.format(~'{0} файлів'~, fcount));
				this.btnGetSize.setVisible(dcount > 0);	
				
				m = 0;
				for (i in modesMul)
						m += i*modesMul[i];												
				
				this.cbgOwner.suspendEvents();
				this.cbgGroup.suspendEvents();
				this.cbgOther.suspendEvents();
				
				this.tfOctalCode.setValue(m);
				
				this.frm.getForm().setValues({
						files: files.length == 1 ? d['name'] : a.join(', '),
						location: path, 
						size: fcount > 0 ? Ext.util.Format.fileSize(size) + (dcount > 0 ? ' +' : '') : ''
				});				
				
				this.cbgOwner.setValue([states[400], states[200], states[100]]);
				this.cbgGroup.setValue([states[40], states[20], states[10]]);
				this.cbgOther.setValue([states[4], states[2], states[1]]);
				
				this.cbgOwner.resumeEvents();
				this.cbgGroup.resumeEvents();
				this.cbgOther.resumeEvents();
						
				this.cbXDirs.setValue();
				this.cbRecursive.setValue();
				this.doLayout();				
  },
		
		initComponent: function(){
				var self = this;
		
				this.cbChange = function(){
						var m, mode = 0, n = 0, i, r;
						m = self.cbgOwner.getValue(); 
						r = [400, 200, 100];
						for (i = 0; i < 3; i++) {
						  mode += r[i]*(m[i] === true)
								n += m[i] === null;
						}
						m = self.cbgGroup.getValue(); 
						r = [40, 20, 10];
						for (i = 0; i < 3; i++) {
						  mode += r[i]*(m[i] === true)
								n += m[i] === null;
						}						
						m = self.cbgOther.getValue(); 
						r = [4, 2, 1];
						for (i = 0; i < 3; i++) {
						  mode += r[i]*(m[i] === true)
								n += m[i] === null;
						}	
						self.tfOctalCode.suspendEvents();
						self.tfOctalCode.setValue(n == 0 && mode > 0 ? mode : '');
						self.tfOctalCode.resumeEvents();
				}
				
				this.fileModeDecode = function(x) {
						var a = {}, y = x % 10;
						Ext.apply(a, {1: y & 1 ? 1 : 0, 2: y & 2 ? 1 : 0, 4: y & 4 ? 1 : 0});
						x = Math.floor(x/10);
						y = x % 10;
						Ext.apply(a, {10: y & 1 ? 1 : 0, 20: y & 2 ? 1 : 0, 40: y & 4 ? 1 : 0});
						x = Math.floor(x/10);
						y = x % 10;
						Ext.apply(a, {100: y & 1 ? 1 : 0, 200: y & 2 ? 1 : 0, 400: y & 4 ? 1 : 0});
						return a;
				}
				
				this.btnGetSize = new Ext.Button({
						text: ~'Обчислити розмір'~,
						style: {marginBottom: '5px', marginLeft: '100px'},
						handler: function() {
								var i, fs = [];
								for (i = 0; i < self.files.length; i++)
										fs.push(self.currentPath + '/' + self.files[i].data['name']);
								Ext.Ajax.request({
										params: {
												cmd: 'du',
												'files[]': fs
										},
										success: function(response){
												var o = response.responseData || Ext.decode(response.responseText);
												self.frm.getForm().setValues({size: Ext.util.Format.fileSize(o.du)});
										}
								})
						}																																																					
				});
				
				this.cbgOwner = new Ext.ux.form.TriCheckboxGroup({
						fieldLabel: ~'Власник'~,
						columns: 3,
						defaults: {
								name: 'm'
						},
						items: [{
								boxLabel: 'r',
								inputValue: 400
						},{
								boxLabel: 'w',
								inputValue: 200
						},{
								boxLabel: 'x',
								inputValue: 100
						}],
						listeners: {
								change: this.cbChange,
						}
				});
				
				this.cbgGroup = new Ext.ux.form.TriCheckboxGroup({
						xtype: 'checkboxgroup',
						fieldLabel: ~'Група'~,
						columns: 3,
						defaults: {
								name: 'm'
						},																		
						items: [{
								boxLabel: 'r',
								inputValue: 40
						},{
								boxLabel: 'w',
								inputValue: 20
						},{
								boxLabel: 'x',
								inputValue: 10
						}],
						listeners: {
								change: this.cbChange,
						}
				});
				
				this.cbgOther = new Ext.ux.form.TriCheckboxGroup({
						xtype: 'checkboxgroup',
						fieldLabel: ~'Інші'~,
						columns: 3,
						defaults: {
								name: 'm'
						},																		
						items: [{
								boxLabel: 'r',
								inputValue: 4
						},{
								boxLabel: 'w',
								inputValue: 2
						},{
								boxLabel: 'x',
								inputValue: 1
						}],
						listeners: {
								change: this.cbChange,
						}
				});
				
				this.tfOctalCode = new Ext.form.NumberField({
						xtype: 'textfield',
						fieldLabel: ~'Вісімковий код'~,
						width: 40,
						minValue: 0,
						listeners: {
							 change: function(tf, m){
										if (m == 0)
										  return;
										m = self.fileModeDecode(m);
										
      				self.cbgOwner.suspendEvents();
				      self.cbgGroup.suspendEvents();
				      self.cbgOther.suspendEvents();
										
										self.cbgOwner.setValue([m[400], m[200], m[100]]);
				      self.cbgGroup.setValue([m[40], m[20], m[10]]);
				      self.cbgOther.setValue([m[4], m[2], m[1]]);
				      
										self.cbgOwner.resumeEvents();
				      self.cbgGroup.resumeEvents();
      				self.cbgOther.resumeEvents();
								}
						}
				});
				
				this.cbXDirs = new Ext.form.Checkbox({
						xtype: 'checkbox',
						fieldLabel: ~'Додати X для директорій'~,				
				});
				
				this.cbRecursive = new Ext.form.Checkbox({
						xtype: 'checkbox',
						fieldLabel: ~'Рекурсивно'~,    
				});
				
				this.frm = new Ext.form.FormPanel({
						frame: true,
						border: false,
						bodyStyle: 'padding: 0 5px',															
						items: [{
								xtype: 'displayfield',
								fieldLabel: ~'Файл'~,
								name: 'files'
						},{
								xtype: 'displayfield',
								fieldLabel: ~'Розташування'~,
								name: 'location'
						},{
								xtype: 'displayfield',
								fieldLabel: ~'Розмір'~,
								name: 'size'
						}, this.btnGetSize, {
								xtype: 'fieldset',
								title: ~'Права'~,
								items: [this.cbgOwner, this.cbgGroup, this.cbgOther, this.tfOctalCode, this.cbXDirs, this.cbRecursive]
						}]									
				});
					
			 var config = {
			   items: [this.frm],
				  buttons: [{
								text: ~'Так'~,
								//disabled: true,
								handler: function(){
									 var s = [], p, m, a, r = ['r', 'w', 'x'], xdirs = self.cbXDirs.getValue();
										
 									a = self.cbgOwner.getValue();
										p = m = '';
										for (i = 0; i < 3; i++) {
										  if (a[i] === true)
												  p += r[i];
												if (a[i] === false)
												  m += r[i];
										}
										if (m)
										  s.push('u-' + m);
										if (p)
										  s.push('u+' + p + (xdirs ? 'X' : ''));
									
										a = self.cbgGroup.getValue();
										p = m = '';
										for (i = 0; i < 3; i++) {
										  if (a[i] === true)
												  p += r[i];
												if (a[i] === false)
												  m += r[i];
										}
										if (m)
										  s.push('g-' + m);
										if (p)
										  s.push('g+' + p + (xdirs ? 'X' : ''));

										a = self.cbgOther.getValue();
										p = m = '';
										for (i = 0; i < 3; i++) {
										  if (a[i] === true)
												  p += r[i];
												if (a[i] === false)
												  m += r[i];
										}
										if (m)
										  s.push('o-' + m);
										if (p)
										  s.push('o+' + p + (xdirs ? 'X' : ''));
										
										s = s.join(',').replace('xX', 'x');
										
								  //files, permissions, recursive					
										self.fireEvent('saveattributes', self.files, s, self.cbRecursive.getValue() ? 1 : 0);		
										self.hide();
								}
						},{
								text: ~'Скасувати'~,
								handler: function(){
										self.hide();
								}
						}]			
				};
    Ext.applyIf(this, config);

    app.AttrWindow.superclass.initComponent.call(this);
				
				this.addEvents('saveattributes');
		},
		
//		onRender: function(){
//    app.AttrWindow.superclass.onRender.apply(this, arguments);
				
//		}						
});